import 'package:health/health.dart';

import 'package:vivanta_connect_flutter/enums/apple_health_activity_type.dart';
import 'package:vivanta_connect_flutter/enums/apple_health_parameter_type.dart';
import 'package:vivanta_connect_flutter/enums/meal_types.dart' as meal_types;
import 'package:vivanta_connect_flutter/helpers/helpers.dart';

class AppleHealth {
  HealthFactory health = HealthFactory(useHealthConnectIfAvailable: true);

  Future<bool> requestAuthorization(
      List<HealthPermission> healthPermissions) async {
    final types =
        healthPermissions.map<HealthDataType>((e) => e.healthDataType).toList();
    final permissions = healthPermissions
        .map<HealthDataAccess>((e) => e.healthDataAccess)
        .toList();
    return health.requestAuthorization(types, permissions: permissions);
  }

  Future<bool> authorizationStatus(HealthDataType dataType) async {
    return health.isDataTypeAvailable(dataType);
  }

  Future<bool> hasPermission(List<HealthPermission> healthPermissions) async {
    final types =
        healthPermissions.map<HealthDataType>((e) => e.healthDataType).toList();
    final permissions = healthPermissions
        .map<HealthDataAccess>((e) => e.healthDataAccess)
        .toList();
    final hasPermission =
        await health.hasPermissions(types, permissions: permissions);
    return hasPermission ?? true;
  }

  Future<List<HealthDataPoint>> getDataPoint(
    List<HealthDataType> types,
    DateTime start,
    DateTime end,
  ) async {
    List<HealthDataPoint> healthData = await health.getHealthDataFromTypes(
      start,
      end,
      types,
    );

    return healthData;
  }

  Future<int> getTotalSteps(DateTime start, DateTime end) async {
    int? steps = await health.getTotalStepsInInterval(start, end);
    return steps ?? 0;
  }

  Future<bool> writeParameter({
    required AppleHealthParameterType parameterType,
    required DateTime start,
    required DateTime end,
    required double value,
  }) async {
    return health.writeHealthData(
      value,
      _getDataTypeFromParameter(parameterType),
      start,
      end,
    );
  }

  Future<bool> writeActivity({
    required AppleHealthActivityType activityType,
    required DateTime start,
    required DateTime end,
    int? calories,
    int? distance,
  }) {
    return health.writeWorkoutData(
      _getWorkoutFromActivity(activityType),
      start,
      end,
      totalEnergyBurned: calories,
      totalDistance: distance,
    );
  }

  Future<bool> writeBloodPressure({
    required int systolic,
    required int diastolic,
    required DateTime start,
    required DateTime end,
  }) {
    return health.writeBloodPressure(systolic, diastolic, start, end);
  }

  Future<bool> writeBloodOxygen({
    required double saturation,
    required DateTime start,
    required DateTime end,
    double flowRate = 0.0,
  }) {
    return health.writeBloodOxygen(saturation, start, end, flowRate: flowRate);
  }

  Future<bool> writeAudiogram({
    required List<double> frequencies,
    required List<double> leftEarSensitivities,
    required List<double> rightEarSensitivities,
    required DateTime start,
    required DateTime end,
  }) {
    return health.writeAudiogram(
      frequencies,
      leftEarSensitivities,
      rightEarSensitivities,
      start,
      end,
    );
  }

  Future<bool> writeMeal({
    double? caloriesConsumed,
    double? carbohydrates,
    double? protein,
    double? fatTotal,
    String? name,
    required meal_types.MealType mealType,
    required DateTime start,
    required DateTime end,
  }) {
    return health.writeMeal(
      start,
      end,
      caloriesConsumed,
      carbohydrates,
      protein,
      fatTotal,
      name,
      mealType.toHealthMealType(),
    );
  }

  static AppleHealthParameterType getParameterFromDataType(
      HealthDataType dataType) {
    switch (dataType) {
      case (HealthDataType.ACTIVE_ENERGY_BURNED):
        return AppleHealthParameterType.activeCalories;
      case (HealthDataType.BASAL_ENERGY_BURNED):
        return AppleHealthParameterType.basalCalories;
      case (HealthDataType.BLOOD_GLUCOSE):
        return AppleHealthParameterType.bloodGlucose;
      case (HealthDataType.BLOOD_OXYGEN):
        return AppleHealthParameterType.bloodOxygen;
      case (HealthDataType.BLOOD_PRESSURE_SYSTOLIC):
        return AppleHealthParameterType.bloodPressureSystloc;
      case (HealthDataType.BLOOD_PRESSURE_DIASTOLIC):
        return AppleHealthParameterType.bloodPressureDiastolic;
      case (HealthDataType.BODY_FAT_PERCENTAGE):
        return AppleHealthParameterType.bodyFatPercentage;
      case (HealthDataType.BODY_MASS_INDEX):
        return AppleHealthParameterType.bmi;
      case (HealthDataType.BODY_TEMPERATURE):
        return AppleHealthParameterType.bodyTemperature;
      case (HealthDataType.HEART_RATE):
        return AppleHealthParameterType.heartRate;
      case (HealthDataType.HEIGHT):
        return AppleHealthParameterType.height;
      case (HealthDataType.RESTING_HEART_RATE):
        return AppleHealthParameterType.restingHeartRate;
      case (HealthDataType.RESPIRATORY_RATE):
        return AppleHealthParameterType.respiratoryRate;
      case (HealthDataType.STEPS):
        return AppleHealthParameterType.steps;
      case (HealthDataType.WEIGHT):
        return AppleHealthParameterType.weight;
      case (HealthDataType.FLIGHTS_CLIMBED):
        return AppleHealthParameterType.flightsClimbed;
      case (HealthDataType.SLEEP_ASLEEP):
        return AppleHealthParameterType.totalSleepDuration;
      case (HealthDataType.SLEEP_DEEP):
        return AppleHealthParameterType.deepSleepDuration;
      case (HealthDataType.SLEEP_REM):
        return AppleHealthParameterType.remSleepDuration;
      case (HealthDataType.SLEEP_IN_BED):
        return AppleHealthParameterType.sleepInBedDuration;
      case (HealthDataType.WATER):
        return AppleHealthParameterType.water;
      case (HealthDataType.ELECTRODERMAL_ACTIVITY):
        return AppleHealthParameterType.electrodermalActivity;
      case (HealthDataType.PERIPHERAL_PERFUSION_INDEX):
        return AppleHealthParameterType.peripheralPerfusionIndex;
      case (HealthDataType.WAIST_CIRCUMFERENCE):
        return AppleHealthParameterType.waistCircumference;
      case (HealthDataType.WALKING_HEART_RATE):
        return AppleHealthParameterType.walkingHeartRate;
      case (HealthDataType.DISTANCE_WALKING_RUNNING):
        return AppleHealthParameterType.distance;
      case (HealthDataType.MINDFULNESS):
        return AppleHealthParameterType.mindfulness;
      case (HealthDataType.EXERCISE_TIME):
        return AppleHealthParameterType.activityTime;
      case (HealthDataType.HEADACHE_NOT_PRESENT):
        return AppleHealthParameterType.headacheNotPresent;
      case (HealthDataType.HEADACHE_MILD):
        return AppleHealthParameterType.headacheMild;
      case (HealthDataType.HEADACHE_MODERATE):
        return AppleHealthParameterType.headacheModerate;
      case (HealthDataType.HEADACHE_SEVERE):
        return AppleHealthParameterType.headacheSevere;
      case (HealthDataType.HEADACHE_UNSPECIFIED):
        return AppleHealthParameterType.headacheUnspecified;
      case (HealthDataType.NUTRITION):
        return AppleHealthParameterType.nutrition;
      default:
        return AppleHealthParameterType.steps;
    }
  }

  HealthDataType _getDataTypeFromParameter(AppleHealthParameterType type) {
    switch (type) {
      case (AppleHealthParameterType.activeCalories):
        return HealthDataType.ACTIVE_ENERGY_BURNED;
      case (AppleHealthParameterType.basalCalories):
        return HealthDataType.BASAL_ENERGY_BURNED;
      case (AppleHealthParameterType.bloodGlucose):
        return HealthDataType.BLOOD_GLUCOSE;
      case (AppleHealthParameterType.bloodOxygen):
        return HealthDataType.BLOOD_OXYGEN;
      case (AppleHealthParameterType.bloodPressureSystloc):
        return HealthDataType.BLOOD_PRESSURE_SYSTOLIC;
      case (AppleHealthParameterType.bloodPressureDiastolic):
        return HealthDataType.BLOOD_PRESSURE_DIASTOLIC;
      case (AppleHealthParameterType.bodyFatPercentage):
        return HealthDataType.BODY_FAT_PERCENTAGE;
      case (AppleHealthParameterType.bmi):
        return HealthDataType.BODY_MASS_INDEX;
      case (AppleHealthParameterType.bodyTemperature):
        return HealthDataType.BODY_TEMPERATURE;
      case (AppleHealthParameterType.heartRate):
        return HealthDataType.HEART_RATE;
      case (AppleHealthParameterType.height):
        return HealthDataType.HEIGHT;
      case (AppleHealthParameterType.restingHeartRate):
        return HealthDataType.RESTING_HEART_RATE;
      case (AppleHealthParameterType.respiratoryRate):
        return HealthDataType.RESPIRATORY_RATE;
      case (AppleHealthParameterType.steps):
        return HealthDataType.STEPS;
      case (AppleHealthParameterType.weight):
        return HealthDataType.WEIGHT;
      case (AppleHealthParameterType.flightsClimbed):
        return HealthDataType.FLIGHTS_CLIMBED;
      case (AppleHealthParameterType.totalSleepDuration):
        return HealthDataType.SLEEP_ASLEEP;
      case (AppleHealthParameterType.deepSleepDuration):
        return HealthDataType.SLEEP_DEEP;
      case (AppleHealthParameterType.remSleepDuration):
        return HealthDataType.SLEEP_REM;
      case (AppleHealthParameterType.sleepInBedDuration):
        return HealthDataType.SLEEP_IN_BED;
      case (AppleHealthParameterType.water):
        return HealthDataType.WATER;
      case (AppleHealthParameterType.electrodermalActivity):
        return HealthDataType.ELECTRODERMAL_ACTIVITY;
      case (AppleHealthParameterType.peripheralPerfusionIndex):
        return HealthDataType.PERIPHERAL_PERFUSION_INDEX;
      case (AppleHealthParameterType.waistCircumference):
        return HealthDataType.WAIST_CIRCUMFERENCE;
      case (AppleHealthParameterType.walkingHeartRate):
        return HealthDataType.WALKING_HEART_RATE;
      case (AppleHealthParameterType.distance):
        return HealthDataType.DISTANCE_WALKING_RUNNING;
      case (AppleHealthParameterType.mindfulness):
        return HealthDataType.MINDFULNESS;
      case (AppleHealthParameterType.activityTime):
        return HealthDataType.EXERCISE_TIME;
      case (AppleHealthParameterType.headacheNotPresent):
        return HealthDataType.HEADACHE_NOT_PRESENT;
      case (AppleHealthParameterType.headacheMild):
        return HealthDataType.HEADACHE_MILD;
      case (AppleHealthParameterType.headacheModerate):
        return HealthDataType.HEADACHE_MODERATE;
      case (AppleHealthParameterType.headacheSevere):
        return HealthDataType.HEADACHE_SEVERE;
      case (AppleHealthParameterType.headacheUnspecified):
        return HealthDataType.HEADACHE_UNSPECIFIED;
      case (AppleHealthParameterType.nutrition):
        return HealthDataType.NUTRITION;
      default:
        return HealthDataType.STEPS;
    }
  }

  HealthWorkoutActivityType _getWorkoutFromActivity(
    AppleHealthActivityType activity,
  ) {
    switch (activity) {
      case AppleHealthActivityType.archery:
        return HealthWorkoutActivityType.ARCHERY;
      case AppleHealthActivityType.badminton:
        return HealthWorkoutActivityType.BADMINTON;
      case AppleHealthActivityType.baseball:
        return HealthWorkoutActivityType.BASEBALL;
      case AppleHealthActivityType.basketball:
        return HealthWorkoutActivityType.BASKETBALL;
      case AppleHealthActivityType.biking:
        return HealthWorkoutActivityType.BIKING;
      case AppleHealthActivityType.boxing:
        return HealthWorkoutActivityType.BOXING;
      case AppleHealthActivityType.cricket:
        return HealthWorkoutActivityType.CRICKET;
      case AppleHealthActivityType.curling:
        return HealthWorkoutActivityType.CURLING;
      case AppleHealthActivityType.elliptical:
        return HealthWorkoutActivityType.ELLIPTICAL;
      case AppleHealthActivityType.fencing:
        return HealthWorkoutActivityType.FENCING;
      case AppleHealthActivityType.americanFootball:
        return HealthWorkoutActivityType.AMERICAN_FOOTBALL;
      case AppleHealthActivityType.australianFootball:
        return HealthWorkoutActivityType.AUSTRALIAN_FOOTBALL;
      case AppleHealthActivityType.soccer:
        return HealthWorkoutActivityType.SOCCER;
      case AppleHealthActivityType.golf:
        return HealthWorkoutActivityType.GOLF;
      case AppleHealthActivityType.gymnastics:
        return HealthWorkoutActivityType.GYMNASTICS;
      case AppleHealthActivityType.handball:
        return HealthWorkoutActivityType.HANDBALL;
      case AppleHealthActivityType.highIntensityIntervalTraining:
        return HealthWorkoutActivityType.HIGH_INTENSITY_INTERVAL_TRAINING;
      case AppleHealthActivityType.hiking:
        return HealthWorkoutActivityType.HIKING;
      case AppleHealthActivityType.hockey:
        return HealthWorkoutActivityType.HOCKEY;
      case AppleHealthActivityType.skating:
        return HealthWorkoutActivityType.SKATING;
      case AppleHealthActivityType.jumpRope:
        return HealthWorkoutActivityType.JUMP_ROPE;
      case AppleHealthActivityType.kickboxing:
        return HealthWorkoutActivityType.KICKBOXING;
      case AppleHealthActivityType.martialArts:
        return HealthWorkoutActivityType.MARTIAL_ARTS;
      case AppleHealthActivityType.pilates:
        return HealthWorkoutActivityType.PILATES;
      case AppleHealthActivityType.racquetball:
        return HealthWorkoutActivityType.RACQUETBALL;
      case AppleHealthActivityType.rowing:
        return HealthWorkoutActivityType.ROWING;
      case AppleHealthActivityType.rugby:
        return HealthWorkoutActivityType.RUGBY;
      case AppleHealthActivityType.running:
        return HealthWorkoutActivityType.RUNNING;
      case AppleHealthActivityType.sailing:
        return HealthWorkoutActivityType.SAILING;
      case AppleHealthActivityType.crossCountrySkiing:
        return HealthWorkoutActivityType.CROSS_COUNTRY_SKIING;
      case AppleHealthActivityType.downhillSkiing:
        return HealthWorkoutActivityType.DOWNHILL_SKIING;
      case AppleHealthActivityType.snowboarding:
        return HealthWorkoutActivityType.SNOWBOARDING;
      case AppleHealthActivityType.softball:
        return HealthWorkoutActivityType.SOFTBALL;
      case AppleHealthActivityType.squash:
        return HealthWorkoutActivityType.SQUASH;
      case AppleHealthActivityType.stairClimbing:
        return HealthWorkoutActivityType.STAIR_CLIMBING;
      case AppleHealthActivityType.swimming:
        return HealthWorkoutActivityType.SWIMMING;
      case AppleHealthActivityType.tableTennis:
        return HealthWorkoutActivityType.TABLE_TENNIS;
      case AppleHealthActivityType.tennis:
        return HealthWorkoutActivityType.TENNIS;
      case AppleHealthActivityType.volleyball:
        return HealthWorkoutActivityType.VOLLEYBALL;
      case AppleHealthActivityType.walking:
        return HealthWorkoutActivityType.WALKING;
      case AppleHealthActivityType.waterPolo:
        return HealthWorkoutActivityType.WATER_POLO;
      case AppleHealthActivityType.yoga:
        return HealthWorkoutActivityType.YOGA;
      case AppleHealthActivityType.bowling:
        return HealthWorkoutActivityType.BOWLING;
      case AppleHealthActivityType.crossTraining:
        return HealthWorkoutActivityType.CROSS_TRAINING;
      case AppleHealthActivityType.trackAndField:
        return HealthWorkoutActivityType.TRACK_AND_FIELD;
      case AppleHealthActivityType.discSports:
        return HealthWorkoutActivityType.DISC_SPORTS;
      case AppleHealthActivityType.lacrosse:
        return HealthWorkoutActivityType.LACROSSE;
      case AppleHealthActivityType.preparationAndRecovery:
        return HealthWorkoutActivityType.PREPARATION_AND_RECOVERY;
      case AppleHealthActivityType.flexibility:
        return HealthWorkoutActivityType.FLEXIBILITY;
      case AppleHealthActivityType.cooldown:
        return HealthWorkoutActivityType.COOLDOWN;
      case AppleHealthActivityType.wheelchairWalkPace:
        return HealthWorkoutActivityType.WHEELCHAIR_WALK_PACE;
      case AppleHealthActivityType.wheelchairRunPace:
        return HealthWorkoutActivityType.WHEELCHAIR_RUN_PACE;
      case AppleHealthActivityType.handCycling:
        return HealthWorkoutActivityType.HAND_CYCLING;
      case AppleHealthActivityType.coreTraining:
        return HealthWorkoutActivityType.CORE_TRAINING;
      case AppleHealthActivityType.functionalStrengthTraining:
        return HealthWorkoutActivityType.FUNCTIONAL_STRENGTH_TRAINING;
      case AppleHealthActivityType.traditionalStrengthTraining:
        return HealthWorkoutActivityType.TRADITIONAL_STRENGTH_TRAINING;
      case AppleHealthActivityType.mixedCardio:
        return HealthWorkoutActivityType.MIXED_CARDIO;
      case AppleHealthActivityType.stairs:
        return HealthWorkoutActivityType.STAIRS;
      case AppleHealthActivityType.stepTraining:
        return HealthWorkoutActivityType.STEP_TRAINING;
      case AppleHealthActivityType.fitnessGaming:
        return HealthWorkoutActivityType.FITNESS_GAMING;
      case AppleHealthActivityType.barre:
        return HealthWorkoutActivityType.BARRE;
      case AppleHealthActivityType.cardioDance:
        return HealthWorkoutActivityType.CARDIO_DANCE;
      case AppleHealthActivityType.socialDance:
        return HealthWorkoutActivityType.SOCIAL_DANCE;
      case AppleHealthActivityType.mindAndBody:
        return HealthWorkoutActivityType.MIND_AND_BODY;
      case AppleHealthActivityType.pickleball:
        return HealthWorkoutActivityType.PICKLEBALL;
      case AppleHealthActivityType.climbing:
        return HealthWorkoutActivityType.CLIMBING;
      case AppleHealthActivityType.equestrianSports:
        return HealthWorkoutActivityType.EQUESTRIAN_SPORTS;
      case AppleHealthActivityType.fishing:
        return HealthWorkoutActivityType.FISHING;
      case AppleHealthActivityType.hunting:
        return HealthWorkoutActivityType.HUNTING;
      case AppleHealthActivityType.play:
        return HealthWorkoutActivityType.PLAY;
      case AppleHealthActivityType.snowSports:
        return HealthWorkoutActivityType.SNOW_SPORTS;
      case AppleHealthActivityType.paddleSports:
        return HealthWorkoutActivityType.PADDLE_SPORTS;
      case AppleHealthActivityType.surfingSports:
        return HealthWorkoutActivityType.SURFING_SPORTS;
      case AppleHealthActivityType.waterFitness:
        return HealthWorkoutActivityType.WATER_FITNESS;
      case AppleHealthActivityType.waterSports:
        return HealthWorkoutActivityType.WATER_SPORTS;
      case AppleHealthActivityType.taiChi:
        return HealthWorkoutActivityType.TAI_CHI;
      case AppleHealthActivityType.wrestling:
        return HealthWorkoutActivityType.WRESTLING;
      default:
        return HealthWorkoutActivityType.OTHER;
    }
  }
}
