import 'package:appcheck/appcheck.dart';
import 'package:flutter_health_connect/flutter_health_connect.dart';
import 'package:health/health.dart';
import 'package:url_launcher/url_launcher.dart';

import 'package:vivanta_connect_flutter/enums/google_health_activity_type.dart';
import 'package:vivanta_connect_flutter/enums/google_health_parameter_type.dart';
import 'package:vivanta_connect_flutter/enums/meal_types.dart' as meal_types;
import 'package:vivanta_connect_flutter/helpers/helpers.dart';

const package = "com.google.android.apps.healthdata";
const googleFit = "com.google.android.apps.fitness";

class GoogleHealthConnect {
  HealthFactory health = HealthFactory(useHealthConnectIfAvailable: true);

  Future<void> install() async {
    await launchUrl(
      Uri.parse("market://details?id=$package"),
      mode: LaunchMode.externalApplication,
    );
  }

  Future<void> installGoogleFit() async {
    await launchUrl(
      Uri.parse("market://details?id=$googleFit"),
      mode: LaunchMode.externalApplication,
    );
  }

  Future<bool> isInstalled() async {
    try {
      final isAppEnabled = await AppCheck.isAppEnabled(package);
      return isAppEnabled;
    } catch (e) {
      return false;
    }
  }

  Future<bool> isGoogleFitInstalled() async {
    try {
      final isAppEnabled = await AppCheck.isAppEnabled(googleFit);
      return isAppEnabled;
    } catch (e) {
      return false;
    }
  }

  Future<void> open() async {
    await HealthConnectFactory.openHealthConnectSettings();
  }

  Future<bool> requestAuthorization(
      List<HealthPermission> healthPermissions) async {
    if (await hasPermission(healthPermissions)) {
      return true;
    }
    final types =
        healthPermissions.map<HealthDataType>((e) => e.healthDataType).toList();
    final permissions = healthPermissions
        .map<HealthDataAccess>((e) => e.healthDataAccess)
        .toList();
    return health.requestAuthorization(types, permissions: permissions);
  }

  Future<bool> authorizationStatus(HealthDataType dataType) async {
    return health.isDataTypeAvailable(dataType);
  }

  Future<bool> hasPermission(List<HealthPermission> healthPermissions) async {
    final types =
        healthPermissions.map<HealthDataType>((e) => e.healthDataType).toList();
    final permissions = healthPermissions
        .map<HealthDataAccess>((e) => e.healthDataAccess)
        .toList();
    final hasPermission =
        await health.hasPermissions(types, permissions: permissions);
    return hasPermission ?? false;
  }

  Future<List<HealthDataPoint>> getDataPoint(
    List<HealthDataType> types,
    DateTime start,
    DateTime end,
  ) async {
    List<HealthDataPoint> healthData = await health.getHealthDataFromTypes(
      start,
      end,
      types,
    );

    return healthData;
  }

  Future<bool> writeParameter({
    required GoogleHealthParameterType parameterType,
    required DateTime start,
    required DateTime end,
    required double value,
  }) async {
    return health.writeHealthData(
      value,
      _getDataTypeFromVivantaParameter(parameterType),
      start,
      end,
    );
  }

  Future<bool> writeActivity({
    required GoogleHealthActivityType activityType,
    required DateTime start,
    required DateTime end,
    int? calories,
    int? distance,
  }) {
    return health.writeWorkoutData(
      _getWorkoutFromVivantaActivity(activityType),
      start,
      end,
      totalEnergyBurned: calories,
      totalDistance: distance,
    );
  }

  Future<bool> writeBloodPressure({
    required int systolic,
    required int diastolic,
    required DateTime start,
    required DateTime end,
  }) {
    return health.writeBloodPressure(systolic, diastolic, start, end);
  }

  Future<bool> writeBloodOxygen({
    required double saturation,
    required DateTime start,
    required DateTime end,
    double flowRate = 0.0,
  }) {
    return health.writeBloodOxygen(saturation, start, end, flowRate: flowRate);
  }

  Future<bool> writeMeal({
    double? caloriesConsumed,
    double? carbohydrates,
    double? protein,
    double? fatTotal,
    String? name,
    required meal_types.MealType mealType,
    required DateTime start,
    required DateTime end,
  }) {
    return health.writeMeal(
      start,
      end,
      caloriesConsumed,
      carbohydrates,
      protein,
      fatTotal,
      name,
      mealType.toHealthMealType(),
    );
  }

  static GoogleHealthParameterType getParameterFromDataType(
      HealthDataType dataType) {
    switch (dataType) {
      case (HealthDataType.ACTIVE_ENERGY_BURNED):
        return GoogleHealthParameterType.activeCalories;
      case (HealthDataType.BASAL_ENERGY_BURNED):
        return GoogleHealthParameterType.basalCalories;
      case (HealthDataType.BLOOD_GLUCOSE):
        return GoogleHealthParameterType.bloodGlucose;
      case (HealthDataType.BLOOD_OXYGEN):
        return GoogleHealthParameterType.bloodOxygen;
      case (HealthDataType.BLOOD_PRESSURE_SYSTOLIC):
        return GoogleHealthParameterType.bloodPressureSystloc;
      case (HealthDataType.BLOOD_PRESSURE_DIASTOLIC):
        return GoogleHealthParameterType.bloodPressureDiastolic;
      case (HealthDataType.BODY_FAT_PERCENTAGE):
        return GoogleHealthParameterType.bodyFatPercentage;
      case (HealthDataType.BODY_MASS_INDEX):
        return GoogleHealthParameterType.bmi;
      case (HealthDataType.BODY_TEMPERATURE):
        return GoogleHealthParameterType.bodyTemperature;
      case (HealthDataType.HEART_RATE):
        return GoogleHealthParameterType.heartRate;
      case (HealthDataType.HEIGHT):
        return GoogleHealthParameterType.height;
      case (HealthDataType.RESTING_HEART_RATE):
        return GoogleHealthParameterType.restingHeartRate;
      case (HealthDataType.RESPIRATORY_RATE):
        return GoogleHealthParameterType.respiratoryRate;
      case (HealthDataType.STEPS):
        return GoogleHealthParameterType.steps;
      case (HealthDataType.WEIGHT):
        return GoogleHealthParameterType.weight;
      case (HealthDataType.FLIGHTS_CLIMBED):
        return GoogleHealthParameterType.flightsClimbed;
      case (HealthDataType.SLEEP_ASLEEP):
        return GoogleHealthParameterType.totalSleepDuration;
      case (HealthDataType.SLEEP_DEEP):
        return GoogleHealthParameterType.deepSleepDuration;
      case (HealthDataType.SLEEP_REM):
        return GoogleHealthParameterType.remSleepDuration;
      case (HealthDataType.WATER):
        return GoogleHealthParameterType.water;
      case (HealthDataType.DISTANCE_WALKING_RUNNING):
        return GoogleHealthParameterType.distance;
      case (HealthDataType.EXERCISE_TIME):
        return GoogleHealthParameterType.activityTime;
      case (HealthDataType.DISTANCE_DELTA):
        return GoogleHealthParameterType.distanceDelta;
      case (HealthDataType.SLEEP_LIGHT):
        return GoogleHealthParameterType.lightSleepDuration;
      case (HealthDataType.SLEEP_OUT_OF_BED):
        return GoogleHealthParameterType.outOfBedSleepDuration;
      case (HealthDataType.SLEEP_SESSION):
        return GoogleHealthParameterType.sleepSessionDuration;
      case (HealthDataType.NUTRITION):
        return GoogleHealthParameterType.nutrition;
      default:
        return GoogleHealthParameterType.steps;
    }
  }

  HealthDataType _getDataTypeFromVivantaParameter(
      GoogleHealthParameterType type) {
    switch (type) {
      case (GoogleHealthParameterType.activeCalories):
        return HealthDataType.ACTIVE_ENERGY_BURNED;
      case (GoogleHealthParameterType.basalCalories):
        return HealthDataType.BASAL_ENERGY_BURNED;
      case (GoogleHealthParameterType.bloodGlucose):
        return HealthDataType.BLOOD_GLUCOSE;
      case (GoogleHealthParameterType.bloodOxygen):
        return HealthDataType.BLOOD_OXYGEN;
      case (GoogleHealthParameterType.bloodPressureSystloc):
        return HealthDataType.BLOOD_PRESSURE_SYSTOLIC;
      case (GoogleHealthParameterType.bloodPressureDiastolic):
        return HealthDataType.BLOOD_PRESSURE_DIASTOLIC;
      case (GoogleHealthParameterType.bodyFatPercentage):
        return HealthDataType.BODY_FAT_PERCENTAGE;
      case (GoogleHealthParameterType.bmi):
        return HealthDataType.BODY_MASS_INDEX;
      case (GoogleHealthParameterType.bodyTemperature):
        return HealthDataType.BODY_TEMPERATURE;
      case (GoogleHealthParameterType.heartRate):
        return HealthDataType.HEART_RATE;
      case (GoogleHealthParameterType.height):
        return HealthDataType.HEIGHT;
      case (GoogleHealthParameterType.restingHeartRate):
        return HealthDataType.RESTING_HEART_RATE;
      case (GoogleHealthParameterType.respiratoryRate):
        return HealthDataType.RESPIRATORY_RATE;
      case (GoogleHealthParameterType.steps):
        return HealthDataType.STEPS;
      case (GoogleHealthParameterType.weight):
        return HealthDataType.WEIGHT;
      case (GoogleHealthParameterType.flightsClimbed):
        return HealthDataType.FLIGHTS_CLIMBED;
      case (GoogleHealthParameterType.totalSleepDuration):
        return HealthDataType.SLEEP_ASLEEP;
      case (GoogleHealthParameterType.lightSleepDuration):
        return HealthDataType.SLEEP_LIGHT;
      case (GoogleHealthParameterType.deepSleepDuration):
        return HealthDataType.SLEEP_DEEP;
      case (GoogleHealthParameterType.remSleepDuration):
        return HealthDataType.SLEEP_REM;
      case (GoogleHealthParameterType.water):
        return HealthDataType.WATER;
      case (GoogleHealthParameterType.distance):
        return HealthDataType.DISTANCE_WALKING_RUNNING;
      case (GoogleHealthParameterType.activityTime):
        return HealthDataType.EXERCISE_TIME;
      case (GoogleHealthParameterType.distanceDelta):
        return HealthDataType.DISTANCE_DELTA;
      case (GoogleHealthParameterType.outOfBedSleepDuration):
        return HealthDataType.SLEEP_OUT_OF_BED;
      case (GoogleHealthParameterType.sleepSessionDuration):
        return HealthDataType.SLEEP_SESSION;
      case (GoogleHealthParameterType.sleepAwakeDuration):
        return HealthDataType.SLEEP_AWAKE;
      case (GoogleHealthParameterType.nutrition):
        return HealthDataType.NUTRITION;
    }
  }

  HealthWorkoutActivityType _getWorkoutFromVivantaActivity(
    GoogleHealthActivityType activity,
  ) {
    switch (activity) {
      case GoogleHealthActivityType.badminton:
        return HealthWorkoutActivityType.BADMINTON;
      case GoogleHealthActivityType.baseball:
        return HealthWorkoutActivityType.BASEBALL;
      case GoogleHealthActivityType.basketball:
        return HealthWorkoutActivityType.BASKETBALL;
      case GoogleHealthActivityType.biking:
        return HealthWorkoutActivityType.BIKING;
      case GoogleHealthActivityType.boxing:
        return HealthWorkoutActivityType.BOXING;
      case GoogleHealthActivityType.cricket:
        return HealthWorkoutActivityType.CRICKET;
      case GoogleHealthActivityType.elliptical:
        return HealthWorkoutActivityType.ELLIPTICAL;
      case GoogleHealthActivityType.fencing:
        return HealthWorkoutActivityType.FENCING;
      case GoogleHealthActivityType.americanFootball:
        return HealthWorkoutActivityType.AMERICAN_FOOTBALL;
      case GoogleHealthActivityType.australianFootball:
        return HealthWorkoutActivityType.AUSTRALIAN_FOOTBALL;
      case GoogleHealthActivityType.golf:
        return HealthWorkoutActivityType.GOLF;
      case GoogleHealthActivityType.gymnastics:
        return HealthWorkoutActivityType.GYMNASTICS;
      case GoogleHealthActivityType.handball:
        return HealthWorkoutActivityType.HANDBALL;
      case GoogleHealthActivityType.highIntensityIntervalTraining:
        return HealthWorkoutActivityType.HIGH_INTENSITY_INTERVAL_TRAINING;
      case GoogleHealthActivityType.hiking:
        return HealthWorkoutActivityType.HIKING;
      case GoogleHealthActivityType.skating:
        return HealthWorkoutActivityType.SKATING;
      case GoogleHealthActivityType.martialArts:
        return HealthWorkoutActivityType.MARTIAL_ARTS;
      case GoogleHealthActivityType.pilates:
        return HealthWorkoutActivityType.PILATES;
      case GoogleHealthActivityType.racquetball:
        return HealthWorkoutActivityType.RACQUETBALL;
      case GoogleHealthActivityType.rowing:
        return HealthWorkoutActivityType.ROWING;
      case GoogleHealthActivityType.rugby:
        return HealthWorkoutActivityType.RUGBY;
      case GoogleHealthActivityType.running:
        return HealthWorkoutActivityType.RUNNING;
      case GoogleHealthActivityType.sailing:
        return HealthWorkoutActivityType.SAILING;
      case GoogleHealthActivityType.snowboarding:
        return HealthWorkoutActivityType.SNOWBOARDING;
      case GoogleHealthActivityType.softball:
        return HealthWorkoutActivityType.SOFTBALL;
      case GoogleHealthActivityType.squash:
        return HealthWorkoutActivityType.SQUASH;
      case GoogleHealthActivityType.stairClimbing:
        return HealthWorkoutActivityType.STAIR_CLIMBING;
      case GoogleHealthActivityType.tableTennis:
        return HealthWorkoutActivityType.TABLE_TENNIS;
      case GoogleHealthActivityType.tennis:
        return HealthWorkoutActivityType.TENNIS;
      case GoogleHealthActivityType.volleyball:
        return HealthWorkoutActivityType.VOLLEYBALL;
      case GoogleHealthActivityType.walking:
        return HealthWorkoutActivityType.WALKING;
      case GoogleHealthActivityType.waterPolo:
        return HealthWorkoutActivityType.WATER_POLO;
      case GoogleHealthActivityType.yoga:
        return HealthWorkoutActivityType.YOGA;
      case GoogleHealthActivityType.calisthenics:
        return HealthWorkoutActivityType.CALISTHENICS;
      case GoogleHealthActivityType.dancing:
        return HealthWorkoutActivityType.DANCING;
      case GoogleHealthActivityType.frisbeeDisc:
        return HealthWorkoutActivityType.FRISBEE_DISC;
      case GoogleHealthActivityType.guidedBreathing:
        return HealthWorkoutActivityType.GUIDED_BREATHING;
      case GoogleHealthActivityType.paragliding:
        return HealthWorkoutActivityType.PARAGLIDING;
      case GoogleHealthActivityType.rockClimbing:
        return HealthWorkoutActivityType.ROCK_CLIMBING;
      case GoogleHealthActivityType.runningTreadmill:
        return HealthWorkoutActivityType.RUNNING_TREADMILL;
      case GoogleHealthActivityType.scubaDiving:
        return HealthWorkoutActivityType.SCUBA_DIVING;
      case GoogleHealthActivityType.stairClimbingMachine:
        return HealthWorkoutActivityType.STAIR_CLIMBING_MACHINE;
      case GoogleHealthActivityType.strengthTraining:
        return HealthWorkoutActivityType.STRENGTH_TRAINING;
      case GoogleHealthActivityType.surfing:
        return HealthWorkoutActivityType.SURFING;
      case GoogleHealthActivityType.swimmingOpenWater:
        return HealthWorkoutActivityType.SWIMMING_OPEN_WATER;
      case GoogleHealthActivityType.swimmingPool:
        return HealthWorkoutActivityType.SWIMMING_POOL;
      case GoogleHealthActivityType.weightlifting:
        return HealthWorkoutActivityType.WEIGHTLIFTING;
      case GoogleHealthActivityType.wheelchair:
        return HealthWorkoutActivityType.WHEELCHAIR;
      default:
        return HealthWorkoutActivityType.OTHER;
    }
  }
}
